/*
    Common commands implementation.
    Copyright (C) 2011 2012  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __CMD_COMMON_C
#include "cmd_common.h"				// functions prototypes

#include <stdio.h>				// NULL
#include <unistd.h>
#include <string.h>
#include "servo.h"				// controller registers

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* macro interne alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int syncCmd(int argc,char **argv)
{
  /* Sincronizzza lo stato del sequencer con lo stato del controller remoto. */



  int	i;



  if(!ServoSyncReg())			//se ha successo
  {
    // stampa lo stato dei registri del controller
    printf("Ctrl  : 0x%04x\n",Controller.Reg[0]);
    for(i=0;i<6;i++)
      printf("Dest  : 0x%04x\n",Controller.Reg[i+1]);
    for(i=0;i<6;i++)
      printf("Speed : 0x%04x\n",Controller.Reg[i+1+6]);
    for(i=0;i<6;i++)
      printf("Pos   : 0x%04x\n",Controller.Reg[i+1+6+6]);
    printf("Offset: 0x%04x\n",Controller.Reg[1+6+6+6]);
    printf("Timing: 0x%04x\n",Controller.Reg[1+6+6+6+1]);
    return 0;				// termina con successo
  }
  return -1;				// termina con fallimento
}



int moveCmd(int argc,char **argv)
{
  /* Muove il servo specificato alla posizione inidcata. */



  unsigned int	Servo;			// servo spostato
  unsigned int	Dest;			// destinazione del servo
  unsigned int	Speed;			// velocita' del servo



  if(argc < 3)				// se mancano degli argomenti
    return 0;				// termina immediatamente
  sscanf(argv[1],"%ui",&Servo);		// legge il servo
  sscanf(argv[2],"%ui",&Dest);		// legge la destinazione
  if(argc > 3)				// se c'e' anche la velocita'
  {
    sscanf(argv[3],"%ui",&Speed);	// legge la velocita'
    {
      // imposta la nuova velocita'
      Controller.Bit.Speed[Servo]=(unsigned char)Speed;
      if(ServoWriteReg(SpeedToReg(Servo)))	// se ci sono problemi
        return -1;			// termina con fallimento
    }
  }

  // imposta la nuova posizione
  Controller.Bit.Dest[Servo]=(unsigned char)Dest;
  if(ServoWriteReg(DestToReg(Servo)))	// se ci sono problemi
    return -1;				// termina con fallimento
  return 0;				// termina con successo
}



int dmoveCmd(int argc,char **argv)
{
  /* Muove una coppia di servi alla posizione specificata. */
  
  
  
  unsigned int	Servo;			// servo spostato
  unsigned int	Dest0;			// destinazione del servo
  unsigned int	Dest1;			// destinazione del servo



  if(argc < 4)				// se mancano degli argomenti
    return 0;				// termina immediatamente
  sscanf(argv[1],"%ui",&Servo);		// legge il servo
  sscanf(argv[2],"%ui",&Dest0);		// legge la destinazione
  sscanf(argv[3],"%ui",&Dest1);		// legge la destinazione

  // imposta la nuova posizione
  Servo&=~0x0001;			// lo rende pari
  Controller.Bit.Dest[Servo]=(unsigned char)Dest0;
  Controller.Bit.Dest[Servo+1]=(unsigned char)Dest1;
  if(ServoWriteReg(DestToReg(Servo)))	// se ci sono problemi
    return -1;				// termina con fallimento
  return 0;				// termina con successo
}



int speedCmd(int argc,char **argv)
{
  /* Imposta la velocita' per il servo indicato. */
  
  
  
  unsigned int	Servo;			// servo spostato
  unsigned int	Speed;			// velocita' del servo
  
  
  
  if(argc < 3)				// se mancano dei parametri
    return 0;				// termina immediatamente
  sscanf(argv[1],"%ui",&Servo);		// legge il servo
  sscanf(argv[2],"%ui",&Speed);		// legge la velocita'

  // imposta la nuova velocita'
  Controller.Bit.Speed[Servo]=(unsigned char)Speed;
  if(ServoWriteReg(SpeedToReg(Servo)))	// se ci sono problemi
    return -1;				// termina con fallimento
  return 0;				// termina con successo
}



int pauseCmd(int argc,char **argv)
{
  /* Esegue una pausa espressa in millisecondi. */



  int	Pause=1000;			// durata dell pausa



  if(argc > 1)				// se c'e' almeno un argomento
    sscanf(argv[1],"%i",&Pause);	// legge la durata della pausa
  usleep(Pause*1000);			// pausa
  return 0;				// termina con successo
}



int waitCmd(int argc,char **argv)
{
  /* Attende che il servo indicato abbia raggiunto la propria destinazione. */
  
  
  
  unsigned int	Servo;			// servo spostato
  
  
  
  if(argc < 2)				// se mancano dei parametri
    return 0;				// termina immediatamente
  sscanf(argv[1],"%ui",&Servo);		// legge il servo

  // attende il raggiungimento della destinaione
  do
  {
    if(ServoReadReg(PosToReg(Servo)))	// legge la posizione attuale
      return -1;			// termina con fallimento
  }while(Controller.Bit.Pos[Servo]!=Controller.Bit.Dest[Servo]);
  return 0;				// termina con successo
}



int timingCmd(int argc,char **argv)
{
  /* Imposta una nuova strategia di timing. */



  unsigned int	Offset;			// offset iniziale
  unsigned int	Timing;			// coefficiente timing
  
  
  
  if(argc < 3)				// se mancano dei parametri
    return 0;				// termina immediatamente
  sscanf(argv[1],"%ui",&Offset);	// legge l'offset iniziale
  sscanf(argv[2],"%ui",&Timing);	// legge il coefficiente timing

  // imposta la nuova temporizzazione
  Controller.Reg[1+6+6+6]=(unsigned short int)Offset;
  Controller.Reg[1+6+6+6+1]=(unsigned short int)Timing;
  if(ServoWriteReg(1+6+6+6))		// se ci sono problemi
    return -1;				// termina con fallimento
  if(ServoWriteReg(1+6+6+6+1))		// se ci sono problemi
    return -1;				// termina con fallimento
  return 0;				// termina con successo
}

/* $Source$ */

