/*
    Command interpreter implementation.
    Copyright (C) 2011  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __COMMAND_C
#include "command.h"				// functions prototypes

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "cmd_def.h"				// command table start,end
#include "cmd_common.h"				// all devices commands

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* macro interne alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static char	CmdName[32];			// nome comando + spazi bianchi

/***************************************************************************/
/* funzioni interne alla libreria */

static int testCmd(int argc,char **argv);
  /* Testa l'interprete dei comandi. */

static int helpCmd(int argc,char **argv);
  /* Visualizza l'help. */

static char *GetCmdName(const char *Name,int TotSpace);
  /* Dato il nome di un comando crea una stringa formata dal comando piu'
     un numero opportuno di spazi fino a arrivare a TotSpace.
     Ritorna un puntatore alla stringa. */

/***************************************************************************/
/* commands list */

static ShellCmd_t	CmdList[]={

ADD_SHELL_CMD(help,"Print command help or commands list",
"[command]\n\n"
"Print the command help, without args print the command list"),

ADD_SHELL_CMD(test,"Command interpreter test",
"<args>\n\n"
"Test the command interpreter"),



ADD_SHELL_CMD(sync,"Syncronize the sequencer with controller",
"\n\n"
"Read back the entire registers set from the controller"),

ADD_SHELL_CMD(move,"Move a single servo",
"servo new_destination <speed>\n\n"
"Set the destination for specified servo"),

ADD_SHELL_CMD(dmove,"Move a couple of servos",
"servo new_destination_0 new_destination_1\n\n"
"Set at once the destination of both servos in the pair"),

ADD_SHELL_CMD(speed,"Set servo speed",
"new_speed\n\n"
"Change the servo speed"),

ADD_SHELL_CMD(pause,"Delay code execution",
"millisec\n\n"
"Delay the program execution for a given number of milliseconds"),

ADD_SHELL_CMD(wait,"Wait for specfied servo",
"servo\n\n"
"Block the program execution until given servo has reached it's destination"),

ADD_SHELL_CMD(timing,"Set new timing strategy",
"offset timing\n\n"
"Change timing registers"),

{NULL,NULL,NULL,NULL}};					// end of list

/***************************************************************************/
/* funzioni */

static int testCmd(int argc,char **argv)
{
  /* Testa l'interprete dei comandi. */



  int	i;



  for(i=0;i<argc;i++)
    printf("argv[%i]=%s\n",i,argv[i]);
  return 0;				// termina con successo
}



static int helpCmd(int argc,char **argv)
{
  /* Visualizza l'help. */



  ShellCmd_t	*Entry;			// puntatore all'entry table



#if SHELL_CMD_SECTION
  Entry=&__shell_cmd_start;		// punta alla prima entry
#else
  Entry=CmdList;			// punta alla prima entry
#endif
  if(argc==1)
  {
    // stampa la lista dei comandi
    puts("\nCommands list:\n--------------");
#if SHELL_CMD_SECTION
    while(Entry!=&__shell_cmd_end)	// fino a fine tabella
#else
    while(Entry->ActFunc != NULL)	// fino al terminatore
#endif
    {
      printf("%s%s\n",GetCmdName(Entry->Cmd,14),Entry->ShortHelp);
      Entry++;
    }
    printf("\n");
  }
  else
  {
    // stampa l'help specifico del comando richiesto
#if SHELL_CMD_SECTION
    while(Entry!=&__shell_cmd_end)	// fino a fine tabella
#else
    while(Entry->ActFunc != NULL)	// fino al terminatore
#endif
    {
      if(strcmp(argv[1],Entry->Cmd)==0)
      {
        printf("Usage: %s %s.\n\n",Entry->Cmd,Entry->LongHelp);
	return 0;			// termina con successo
      }
      Entry++;				// entry successiva
    }
    printf("Unknown command: %s\n",argv[1]);
  }
  return 0;				// termina con successo
}



static char *GetCmdName(const char *Name,int TotSpace)
{
  /* Dato il nome di un comando crea una stringa formata dal comando piu'
     un numero opportuno di spazi fino a arrivare a TotSpace.
     Ritorna un puntatore alla stringa. */



  int	Len;				// lunghezza originale comando



  Len=strlen(Name);			// getta la lunghezza originale
  strcpy(CmdName,Name);			// copia il nome originale
  while(Len < TotSpace)
    CmdName[Len++]=' ';			// aggiunge uno spazio vuoto
  CmdName[Len]=0x00;			// termina la stringa
  return(CmdName);			// termina
}



int ExecCmd(int argc,char **argv)
{
  /* Cerca il comando nella lista comandi e lo esegue, ritorna 0 normalmente,
     ritorna 1 per chiedere la terminazione del processo. */
    
  

  ShellCmd_t	*Entry;			// puntatore all'entry table


    
  if(strcmp(argv[0],"exit")==0)		// se il comando e' 'exit'
    return 1;				// richiede la terminazione
  if(strcmp(argv[0],"quit")==0)		// se il comando e' 'quit'
    return 1;				// richiede la terminazione

  // cerca il comando nella lista comandi
#if SHELL_CMD_SECTION
  Entry=&__shell_cmd_start;		// punta alla prima entry
  while(Entry!=&__shell_cmd_end)	// fino a fine tabella
#else
  Entry=CmdList;			// punta alla prima entry
  while(Entry->ActFunc != NULL)		// fino al terminatore
#endif
  {
    if(strcmp(argv[0],Entry->Cmd)==0)
    {
      if(Entry->ActFunc(argc,argv))	// esegue il comando
	//printf("Usage: %s %s.\n\n",Entry->Cmd,Entry->LongHelp);
	puts("Comunication error");
      return 0;				// termina con successo
    }
    Entry++;				// prossima entry della lista
  }
  printf("Unknown command: %s\n",argv[0]);
  return 0;				// termina con successo
}

/* $Source$ */

