/*
    Servo controller registers access functions.
    Copyright (C) 2012  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __SERVO_C
#include "servo.h"				// functions prototypes

#include <modbus/modbus.h>

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static modbus_param_t		Link;		// modbus link desc.

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int ServoInit(const char *Device)
{
  /* Inizializza la porta seriale e tutte le strutture necessarie,
     ritorna 0 in caso di successo. */
  


  modbus_init_rtu(&Link,Device,2400,"none",8,1);
  return(modbus_connect(&Link));	// apre la seriale
}



int ServoClose(void)
{
  /* Dealloca le risorse necessarie e chiude la porta seriale,
     ritorna 0 in cso di successo. */
  
  
  
  modbus_close(&Link);			// chiude la seriale
  return 0;				// termina con successo
}



int ServoReadReg(unsigned int RegIdx)
{
  /* Aggiorna nel sequencer il registro indicato, ritorna 0 in caso
     di successo. */
  
  
  
  int	Retry=5;			// contatore tentativi



  while(read_holding_registers(&Link,0x01,
  				0x0060+(RegIdx<<1),1,
				&Controller.Reg[RegIdx])<0)
  {
    Retry--;				// scala un tentativo
    if(Retry)				// se sono finiti
      return -1;			// termina con fallimento
  }
  return 0;				// termina con successo
}



int ServoSyncReg(void)
{
  /* Legge tutti i registri del controller e aggiorna il sequencer. */



  int	i;



  for(i=0;i<1+6+6+6+2;i++)
  {
    if(ServoReadReg(i))
      return -1;			// termina con fallimento
  }
  return 0;				// termina con successo
}



int ServoWriteReg(unsigned int RegIdx)
{
  /* Aggiorna nel device remoto il registro indicato, ritorna 0 in caso
     di successo. */



  int	Retry=5;			// contatore tentativi



  while(preset_single_register(&Link,0x01,
  				0x0060+(RegIdx<<1),
				Controller.Reg[RegIdx])<0)
  {
    Retry--;				// scala un tentativo
    if(Retry)				// se sono finiti
      return -1;			// termina con fallimento
  }
  return 0;				// termina con successo
}

/* $Source$ */

