/*
    Servo controller registers access functions.
    Copyright (C) 2012  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

#ifndef __SERVO_H
#define __SERVO_H	1


/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* macro pubbliche */

#define DestToReg(i)		((i>>1)+1)
#define SpeedToReg(i)		((i>>1)+1+6)
#define PosToReg(i)		((i>>1)+1+6+6)

/***************************************************************************/
/* strutture pubbliche */

typedef union
{
  unsigned short int	Reg[1+6+6+6+2];		// array di registri
  struct
  {
    unsigned short int	Ctrl;			// registro di controllo
    unsigned char	Dest[12];		// array destinazioni
    unsigned char	Speed[12];		// array velocita'
    unsigned char	Pos[12];		// array posizioni
    unsigned short int	Offset;			// registro offset
    unsigned short int	Timing;			// registro timing
  }Bit;
}RegArray_t;

/***************************************************************************/
/* costanti pubbliche */

#ifdef __SERVO_C

#else

#endif

/***************************************************************************/
/* variabili pubbliche */

#ifdef __SERVO_C

RegArray_t		Controller;		// registri del controller

#else

extern RegArray_t	Controller;		// registri del controller

#endif

/***************************************************************************/
/* funzioni pubbliche */

int ServoInit(const char *Device);
  /* Inizializza la porta seriale e tutte le strutture necessarie,
     ritorna 0 in caso di successo. */

int ServoClose(void);
  /* Dealloca le risorse necessarie e chiude la porta seriale,
     ritorna 0 in cso di successo. */


#ifndef __INIT_ONLY_FNCT

int ServoReadReg(unsigned int RegIdx);
  /* Aggiorna nel sequencer il registro indicato, ritorna 0 in caso
     di successo. */

int ServoSyncReg(void);
  /* Legge tutti i registri del controller e aggiorna il sequencer. */

int ServoWriteReg(unsigned int RegIdx);
  /* Aggiorna nel device remoto il registro indicato, ritorna 0 in caso
     di successo. */

#endif

/***************************************************************************/
#endif

/* $Source$ */

