/*
    Servo sequencer.
    Copyright (C) 2010 2011 2012 2013  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "parser.h"			// command parser
#include "command.h"			// commands execution
#include "servo.h"			// controller comuniation

/***************************************************************************/
/* define vari */

#define PDEBUG(fmt,args...)	printf(fmt,## args)

#define MAX_ARGV	16

/***************************************************************************/
/* macro */

/***************************************************************************/
/* strutture */

/***************************************************************************/
/* costanti */

static const char Licenza[]=
	"servoseq version 0.02, Copyright (C) 2012 2013  Alessandro Accattini\n";
static const char Help[]=
	"Usage: servoseq [OPTIONS] [FILE1] [FILE2] ...\n"
	"servoseq is a servos sequencer, it reads commands from FILE and\n"
	"comunicates directly with remote servo controller card.\n"
	"If FILE is not specified, servoseq runs in interactive mode.\n"
	"\t-d DEV\tuse DEV as serial port.\n"
	"\t-h\tdisplay this help.\n"
	"\t-l\tenable loop mode (repeat indefinitely).\n"
	"\t-r N\trepeat N times all files.\n"
	"\nReport bugs to bit.trapper@gmail.com\n";

static char	Prompt[]="servoseq> ";	// prompt dei comandi

/***************************************************************************/
/* variabili */

char	*argv[MAX_ARGV];		// argomenti del comando
int	argc;				// numero argomenti
char	InBuf[256];			// input da tastiera

/***************************************************************************/
/* prototipi funzioni */

void ReadFile(FILE *Input,int Interactive);
  /* Legge ed interpreta il file indicato, termina alla fine del file o
     all'esecuzione del comando exit. */

/***************************************************************************/
/* funzioni */

void ReadFile(FILE *Input,int Interactive)
{
  /* Legge ed interpreta il file indicato, termina alla fine del file o
     all'esecuzione del comando exit. */
  
  
  
  int	Len;				// lunghezza stringa
  int	Argc;				// numero di argomenti per comando
  

  
  while(1)
  {
    if(Interactive)			// se l'input e' da tastiera
      printf("%s",Prompt);		// stampa il prompt dei comandi
    if(fgets(InBuf,256,Input)==NULL)	// legge una stringa dal file
      return;				// termina la lettura del file
    if(!Interactive)			// se l'input e' da file
      printf(InBuf);			// stampa la riga in ingresso
    Len=strlen(InBuf)-1;		// lunghezza stringa
    if(InBuf[Len] == '\n')		// se finisce con ENTER
      InBuf[Len]=0x00;			// termina prima
    if((Argc=ParseLine(InBuf)))		// crea Argc e Argv
    {
      if(ExecCmd(Argc,Argv))		// esegue il comando digitato
	return;				// abbandona la lettura del file
    }
  }
}





/***************************************************************************/

int main(int argc,char **argv)
{
  FILE	*Input;				// input file desc
  char	*Device="/dev/ttyUSB0";		// device seriale
  char	*VarValue;			// valore variabile d'ambiente
  int	Opzione;			// per la lettura delle opzioni
  int	Repeat=1;			// repeat counter
  int	Loop=0;				// loop-mode flag
  int	i;



  // legge le variabili d'ambiente
  if((VarValue=getenv("SERVOSEQ_DEV"))!=NULL)
    Device=VarValue;

  // visualizza la licenza del programma
  puts(Licenza);
  
  // legge le opzioni presenti sulla riga di comando
  while((Opzione=getopt(argc,argv,"d:hlr:"))!=-1)
  {
    switch(Opzione)
    {
      case 'd':
        Device=optarg;			// cambia il device aperto
      break;
      case 'h':
        puts(Help);			// stampa l'help
	return 0;			// termina
      case 'l':
        Loop=1;				// enable loop mode
      break;
      case 'r':
        sscanf(optarg,"%i",&Repeat);	// imposta il numero di ripetizioni
      break;
      default:
        puts("Unknown option, use -h to see help.");
	return 0;			// termina
    }
  }

  // apre il controllo dell'interfacia hardware
  if(ServoInit(Device))			// se non riesce ad aprire la seriale
    return 1;				// termina con fallimento

  // main loop
  if(optind < argc)			// se ci sono altri argomenti
  {
    // proessa tutti i file indicati
    do
    {
      for(i=optind;i<argc;i++)
      {
        if((Input=fopen(argv[i],"r"))==NULL)
	{
	  perror("fopen");		// stampa l'errore
	  continue;			// next file
	}
	ReadFile(Input,0);		// input da file
	fclose(Input);			// chiude l'input stream
      }
      Repeat--;				// scala le ripetizioni
    }while(Loop || Repeat);
  }
  else
  {
    // modalita' interattiva
    ReadFile(stdin,1);			// input da tastiera
  }
  
  ServoClose();				// chiude la seriale
  puts("\n\nThank you for using 'servoseq'!\n");
  return 0;				// termina
}

/* $Source$ */

